@@ -27,10 +27,11 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <fcntl.h>
-#include <X11/Xatom.h>
 #include <unistd.h>
 
+#define DEFAULT_SAVE_DIRECTORY xfce_get_homedir ()
+
+/* Struct to store the screenshot options */
 typedef struct
 {
   gint whole_screen;
@@ -41,5 +42,5 @@ typedef struct
 }
 ScreenshotData;
 
-GdkPixbuf *take_screenshot (ScreenshotData * sd);
-void save_screenshot (GdkPixbuf * screenshot, ScreenshotData * sd);
+GdkPixbuf *take_screenshot (ScreenshotData *sd);
+void save_screenshot (GdkPixbuf *screenshot, ScreenshotData *sd);
